/***************************************************************************
 *   Copyright (C) 2004, 2005 by Gerd Fleischer                            *
 *   gerdfleischer@web.de                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//#define MY_EXTRA_DEBUG

#include <kdebug.h>

#include <qfile.h>
#include <qlabel.h>
#include <qstringlist.h>
#include <qregexp.h>

#include <kaction.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kpushbutton.h>
#include <kuser.h>

#include "makepopup.h"
#include "klinpopup.h"

void readGroupsThread::run()
{
    threadOwner->readGroupList();
}

void readHostsThread::run()
{
    threadOwner->readHostList();
}

makePopup::makePopup(QWidget *parent, const char *name, const QString &paramSender,
                     const QString &paramSmbclient, int paramEncoding)
    : makePopup_view(parent, name, WType_TopLevel),
      smbclientBin(paramSmbclient),
      newMsgEncoding(paramEncoding),
      p(NULL), readHosts(this), readGroups(this)
{
    messageText->installEventFilter(this);

    initSmbCtx();

    //initialize senderBox, groupBox and hostBox
    QString tmpHostName = smbCtx->netbios_name;
    if (!tmpHostName.isEmpty()) senderBox->insertItem(tmpHostName);

    QString tmpLoginName = KUser().loginName();
    if (!tmpLoginName.isEmpty()) senderBox->insertItem(tmpLoginName);

    QString tmpFullName = KUser().fullName();
    if (!tmpFullName.isEmpty()) senderBox->insertItem(tmpFullName);

    if (paramSender.isEmpty()) {
        groupBox->insertItem(i18n("NO GROUP"), -1);
        readGroups.start();
    } else {
        groupBox->setEnabled(false);
        hostBox->insertItem(paramSender);
        hostBox->setEnabled(false);
    }

    connect(groupBox, SIGNAL(activated(const QString &)), this, SLOT(slotGroupboxChanged(const QString &)));
    connect(buttonSend, SIGNAL(clicked()), this, SLOT(slotButtonSend()));
    connect(buttonCancel, SIGNAL(clicked()), this, SLOT(finished()));

    messageText->setFocus();
}


makePopup::~makePopup()
{
}

void makePopup::closeEvent(QCloseEvent *e)
{
    e->ignore();
    finished();
}

/**
 * filter keypresses and restrict message size to 1600 bytes
 */
bool makePopup::eventFilter(QObject *, QEvent *e)
{
    if (e->type() == QEvent::KeyPress) {
        QKeyEvent *k = (QKeyEvent *)e;
        //do funny things to figure out the bytes of the message and react accordingly
        if ((k->key() >= Qt::Key_Space && k->key() <= Qt::Key_ydiaeresis) || k->key() == Qt::Key_Tab) {
            if ((messageText->length() + messageText->lines()) > 1600) return true;
        } else if (k->key() == Qt::Key_Return) {
            if (((messageText->length() + 1) + messageText->lines()) > 1600) return true;
        }
    }

    return false;
}

/**
 * hide and destroy when message has been sent or canceled
 */
void makePopup::finished()
{
    hide();

    //prevent crashes?
    delete p;
    p = NULL;

    // This prevents a crash if closing while a thread is running
    // but may hang if one catch libsmbclient in a bad moment. :-(

    // This should be safe here
    if (readGroups.running()) readGroups.terminate();
    if (readHosts.running()) readHosts.terminate();

    readGroups.wait();
    readHosts.wait();

    smbc_free_context(smbCtx, 1);

    destroy();
    emit makePopupEnd(this);
}

/**
 * called when sendButton emits clicked()
 * checks for empty hostbox and sends message
 */
void makePopup::slotButtonSend()
{
    QString tmpHostBox = hostBox->currentText();
    if (tmpHostBox.isEmpty()) {
       KMessageBox::information(this, i18n("Please state an addressee!"));
       hostBox->setFocus();
    } else {
       sendPopup();
    }
}

/**
 * sends the message with smbclient, opens a KProcess for this
 */
void makePopup::sendPopup()
{
    QString tmpReceiver = hostBox->currentText();
    if (tmpReceiver.find(" (") > 0) tmpReceiver = tmpReceiver.left(tmpReceiver.find(" ("));

    p = new KProcess(this);
    *p << smbclientBin << "-M" << tmpReceiver;
    *p << "-N" << "-U" << senderBox->currentText() << "-";

    connect(p, SIGNAL(processExited(KProcess *)),
            this, SLOT(slotSendCmdExit(KProcess *)));

    if (p->start(KProcess::NotifyOnExit, KProcess::Stdin)) {
        QString tmpText = messageText->text();

///@TODO: does local8Bit work for all environments?
        switch (newMsgEncoding)
        {
            case ENC_LOCAL8BIT:
                p->writeStdin(tmpText.local8Bit(), tmpText.local8Bit().length());
                break;
            case ENC_UTF8:
                p->writeStdin(tmpText.utf8(), tmpText.utf8().length());
                break;
            case ENC_LATIN1:
                p->writeStdin(tmpText.latin1(), tmpText.length());
                break;
            case ENC_ASCII:
                p->writeStdin(tmpText.ascii(), tmpText.length());
                break;
            default:
                p->writeStdin(tmpText.local8Bit(), tmpText.local8Bit().length());
                break;
        }
        if (!p->closeStdin()) {
///@TODO: does this work, how to test this?
            delete p;
            p = NULL;
        }
    } else
        slotSendCmdExit(NULL);
}

void makePopup::slotSendCmdExit(KProcess *_p)
{
    if (_p && _p->normalExit() && _p->exitStatus() == 0){
        KMessageBox::information(this, i18n("Message sent!"), i18n("Success"), "ShowMessageSentSuccess");
        delete p;
        p = NULL;
        finished();
    } else {
        int tmpYesNo = KMessageBox::warningYesNo(this, i18n("Message could not be sent!\n"
                                                            "Edit/Try again?"));
        if (tmpYesNo != KMessageBox::Yes) {
            delete p;
            p = NULL;
            finished();
        } else {
            delete p;
            p = NULL;
        }
    }
}

/**
 * init the libsmbclient context
 */
void makePopup::initSmbCtx()
{
    smbCtx = smbc_new_context();
    if (smbCtx) {
        smbCtx->callbacks.auth_fn = ::auth_smbc_get_data;
//        smbCtx->callbacks.get_cached_srv_fn = ::smbc_get_cached_server;
        smbCtx = smbc_init_context(smbCtx);
        smbCtx->timeout = 2000;
    } // else what?
}

/**
 * read available groups with libsmbclient
 */
void makePopup::readGroupList()
{
    QString ownGroup = QString::null;
    bool foundOwnMaster = false;

    if (smbCtx != NULL) {
        // get own workgoup first
        ownGroup = QString::fromUtf8(smbCtx->workgroup, -1);

        SMBCFILE *dirfd;
        struct smbc_dirent *dirp = NULL;

        // first level should be the workgroups
        dirfd = smbCtx->opendir(smbCtx, "smb://");

        if (dirfd >= 0) {
            do {
                dirp = smbCtx->readdir(smbCtx, dirfd);
                if (dirp == 0) break;
                QString tmpGroup = QString::fromUtf8( dirp->name );

                if (dirp->smbc_type == SMBC_WORKGROUP) {
                    if (tmpGroup == ownGroup) {
                        groupBox->insertItem(tmpGroup + " " + i18n("(own)"), -1);
                        foundOwnMaster = true;
                    } else {
                        groupBox->insertItem(tmpGroup, -1);
                    }
                }
            } while (dirp);
        }
        smbCtx->closedir(smbCtx, dirfd);
    }

    // initialize with own group if possible
    if (!ownGroup.isEmpty() && foundOwnMaster) {
        groupBox->setCurrentText(ownGroup + " " + i18n("(own)"));
        readHosts.start();
    }
}

/**
 * read available hosts from group with libsmbclient
 */
void makePopup::readHostList()
{
    hostBox->clear();
    QString group = groupBox->currentText();
    if (group == i18n("NO GROUP")) return;
    if (group.find(" (") > 0) group = group.left(group.find(" ("));

    if (smbCtx != NULL) {


        SMBCFILE *dirfd;
        struct smbc_dirent *dirp = NULL;

        // next level should be the hosts
        QString tmpGroup = "smb://";
        tmpGroup.append(group);
        dirfd = smbCtx->opendir(smbCtx, tmpGroup);

        if (dirfd >= 0) {
            do {
                dirp = smbCtx->readdir(smbCtx, dirfd);
                if (dirp == 0) break;

//                SMBCSRV *test;
//                if (dirp->smbc_type == SMBC_SERVER) {
//                    test = smbCtx->callbacks.get_cached_srv_fn(smbCtx, dirp->name, NULL, NULL, NULL);
//                    kdDebug() << "SMBCSRV: " << test << endl;
//                    smbCtx->callbacks.check_server_fn(smbCtx, dirp);
//                }

                QString tmpHost = QString::fromUtf8(dirp->name);

                if (dirp->smbc_type == SMBC_SERVER) {
                    QString tmpComment = QString::fromUtf8(dirp->comment);
                    if (!tmpComment.isEmpty()) tmpHost.append(" (" + tmpComment + ")");
                    hostBox->insertItem(tmpHost, -1);
                }
            } while (dirp);
        }
        smbCtx->closedir(smbCtx, dirfd);
    }
}

/**
 * will be called if groupbox is changed
 */
void makePopup::slotGroupboxChanged(const QString &)
{
        if (readHosts.running()) readHosts.exit();
        readHosts.start();
}
